#!/usr/bin/python
#
# Copyright (C) 2015  Custodia project Contributors, for licensee see COPYING

from setuptools import setup

requirements = [
    'cryptography',
    'jwcrypto',
    'six',
    'requests']
# configparser; python_version<'3.4'
# extended interpolation is provided by stdlib in Python 3.4+

# extra requirements
etcd_requires = ['python-etcd']

# test requirements
test_requires = ['coverage', 'pytest'] + etcd_requires
test_pylint_requires = ['pylint'] + test_requires
test_pep8_requires = ['flake8', 'flake8-import-order', 'pep8-naming']
test_docs_requires = ['docutils', 'markdown'] + etcd_requires

with open('README') as f:
    long_description = f.read()


# Plugins
custodia_authenticators = [
    'SimpleCredsAuth = custodia.httpd.authenticators:SimpleCredsAuth',
    'SimpleHeaderAuth = custodia.httpd.authenticators:SimpleHeaderAuth',
    'SimpleAuthKeys = custodia.httpd.authenticators:SimpleAuthKeys',
    ('SimpleClientCertAuth = '
     'custodia.httpd.authenticators:SimpleClientCertAuth'),
]

custodia_authorizers = [
    'SimplePathAuthz = custodia.httpd.authorizers:SimplePathAuthz',
    'UserNameSpace = custodia.httpd.authorizers:UserNameSpace',
    'KEMKeysStore = custodia.message.kem:KEMKeysStore',
]

custodia_clients = [
    'KEMClient = custodia.client:CustodiaKEMClient',
    'SimpleClient = custodia.client:CustodiaSimpleClient',
]

custodia_consumers = [
    'Forwarder = custodia.forwarder:Forwarder',
    'Secrets = custodia.secrets:Secrets',
    'Root = custodia.root:Root',
]

custodia_stores = [
    'EncryptedOverlay = custodia.store.encgen:EncryptedOverlay',
    'EncryptedStore = custodia.store.enclite:EncryptedStore',
    'EtcdStore = custodia.store.etcdstore:EtcdStore',
    'SqliteStore = custodia.store.sqlite:SqliteStore',
]


setup(
    name='custodia',
    description='A service to manage, retrieve and store secrets',
    long_description=long_description,
    version='0.3.0',
    license='GPLv3+',
    maintainer='Custodia project Contributors',
    maintainer_email='simo@redhat.com',
    url='https://github.com/latchset/custodia',
    namespace_packages=['custodia'],
    packages=[
        'custodia',
        'custodia.cli',
        'custodia.httpd',
        'custodia.message',
        'custodia.server',
        'custodia.store',
    ],
    entry_points={
        'console_scripts': [
            'custodia = custodia.server:main',
            'custodia-cli = custodia.cli:main',
        ],
        'custodia.authenticators': custodia_authenticators,
        'custodia.authorizers': custodia_authorizers,
        'custodia.clients': custodia_clients,
        'custodia.consumers': custodia_consumers,
        'custodia.stores': custodia_stores,
    },
    classifiers=[
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Intended Audience :: Developers',
        'Topic :: Security',
        'Topic :: Software Development :: Libraries :: Python Modules'
    ],
    install_requires=requirements,
    tests_require=test_requires,
    extras_require={
        # extended interpolation is provided by stdlib in Python 3.4+
        ':python_version<"3.4"': ['configparser'],
        'etcd_store': etcd_requires,
        'test': test_requires,
        'test_docs': test_docs_requires,
        'test_pep8': test_pep8_requires,
        'test_pylint': test_pylint_requires,
    },
)



# Copyright (C) 2016  Custodia Project Contributors - see LICENSE file

import yaml

from requests.exceptions import HTTPError
from custodia.client import CustodiaSimpleClient


class CustodiaSimpleConstructor(object):
    yaml_tag = u'!custodia/simple'

    def __init__(self, url):
        self.client = CustodiaSimpleClient(url)

    def __call__(self, loader, node):
        value = loader.construct_scalar(node)
        return self.client.get_secret(value)


def demo():
    constructor = CustodiaSimpleConstructor(
        'http+unix://%2E%2Fserver_socket/secrets'
    )
    constructor.client.headers['REMOTE_USER'] = 'user'

    # create entries
    try:
        c = constructor.client.list_container('test')
    except HTTPError:
        constructor.client.create_container('test')
        c = []
    if 'key' not in c:
        constructor.client.set_secret('test/key', 'secret password')

    yaml.add_constructor(CustodiaSimpleConstructor.yaml_tag,
                         constructor)
    yaml_str = 'password: !custodia/simple test/key'
    print(yaml_str)
    result = yaml.load(yaml_str)
    print(result)


if __name__ == '__main__':
    demo()



# Copyright (C) 2015  Custodia Project Contributors - see LICENSE file

from custodia.message.common import InvalidMessage
from custodia.message.common import UnallowedMessage
from custodia.message.common import UnknownMessageType
from custodia.message.kem import KEMHandler
from custodia.message.simple import SimpleKey


default_types = ['simple', 'kem']

key_types = {'simple': SimpleKey,
             'kem': KEMHandler}


class Validator(object):
    """Validates incoming messages."""

    def __init__(self, allowed=None):
        """Creates a Validator object.

        :param allowed: list of allowed message types (optional)
        """
        self.allowed = allowed or default_types
        self.types = key_types.copy()

    def add_types(self, types):
        self.types.update(types)

    def parse(self, request, msg, name):
        if not isinstance(msg, dict):
            raise InvalidMessage('The message must be a dict')

        if 'type' not in msg:
            raise InvalidMessage('The type is missing')

        if isinstance(msg['type'], list):
            if len(msg['type']) != 1:
                raise InvalidMessage('Type is multivalued: %s' % msg['type'])
            msg_type = msg['type'][0]
        else:
            msg_type = msg['type']

        if 'value' not in msg:
            raise InvalidMessage('The value is missing')

        if isinstance(msg['value'], list):
            if len(msg['value']) != 1:
                raise InvalidMessage('Value is multivalued: %s' % msg['value'])
            msg_value = msg['value'][0]
        else:
            msg_value = msg['value']

        if msg_type not in list(self.types.keys()):
            raise UnknownMessageType("Type '%s' is unknown" % msg_type)

        if msg_type not in self.allowed:
            raise UnallowedMessage("Message type '%s' not allowed" % (
                                   msg_type,))

        handler = self.types[msg_type](request)
        handler.parse(msg_value, name)
        return handler
